﻿using Kivii.EmailPools.Entities;
using Kivii.Linq;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Net.Mail;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.EmailPools
{
    public static  class Extension
    {
        public static List<EmailPool<G>> Accept<G>(this List<EmailPool> items, IDbConnection conn = null)
        {
            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<EmailPool<G>>();
            var rtns = new List<EmailPool<G>>();
            foreach (var item in items)
            {
                try
                {
                    try
                    {
                        var to = new MailAddress(item.ToAddress, item.ToName);
                    }
                    catch
                    {
                        throw new Exception($"收件地址{item.ToAddress},格式错误");
                    }
                    EmailPool<G> emailPool = new EmailPool<G>();
                    emailPool.PopulateWith(item);
                    conn.Insert(emailPool);
                    rtns.Add(emailPool);
                }
                catch
                {
                    continue;
                }
            }

            return rtns;
        }

        public static List<EmailPool<G>> GetPreSending<G>(this IDbConnection conn)
        {
            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<EmailPool<G>>();
            var rtns = conn.Select<EmailPool<G>>(o => o.IsSended == false);
            return rtns;
        }

        public static List<EmailPool<G>> SendComplete<G>(this List<EmailPool<G>> items, string operatorName=null, Guid? operatorKvid=null, IDbConnection conn = null)
        {
            var rtns = new List<EmailPool<G>>();
            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<EmailPool<G>>();
            if (operatorName.IsNullOrEmpty())
            {
                operatorName = KiviiContext.CurrentMember.FullName;
                operatorKvid = KiviiContext.CurrentMember.Kvid;
            }
            foreach (var item in items)
            {
                item.IsSended = true;
                item.AddOnlyProperties(o => o.IsSended);
                item.OperatorName = operatorName;
                item.AddOnlyProperties(o => o.OperatorName);
                if (operatorKvid != null)
                {
                    item.OperatorKvid = operatorKvid.Value;
                    item.AddOnlyProperties(o => o.OperatorKvid);
                }
                item.Category = "发送成功";
                item.AddOnlyProperties(o => o.Category);
                //item.Remark = "";
                //item.AddOnlyProperties(o => o.Remark);
                conn.UpdateOnly(item);
                rtns.Add(item);

                var log = new EntityLog<G>();
                log.OwnerKvid = item.OwnerKvid;
                log.BizId = item.OwnerName;
                log.BizKvid = item.OwnerKvid;
                log.BizType = typeof(G).FullName;
                log.Type = "电子邮箱";
                log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                log.Summary = "电子邮箱发送完成";
                log.Remark = $"[发送成功] 已成功将{item.OwnerName} 发送至客户邮箱({item.ToAddress}). 时间:{DateTime.Now:yyyy-MM-dd HH:mm:ss}!";
                conn.Insert(log);
            }

            return rtns;
        }

        public static List<EmailPool<G>> SendFail<G>(this List<EmailPool<G>> items, string operatorName = null, Guid? operatorKvid = null, IDbConnection conn = null)
        {
            var rtns = new List<EmailPool<G>>();
            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<EmailPool<G>>();
            if (operatorName.IsNullOrEmpty())
            {
                operatorName = KiviiContext.CurrentMember.FullName;
                operatorKvid = KiviiContext.CurrentMember.Kvid;
            }
            foreach (var item in items)
            {
                item.IsSended = true;
                item.AddOnlyProperties(o => o.IsSended);
                item.OperatorName = operatorName;
                item.AddOnlyProperties(o => o.OperatorName);
                if (operatorKvid != null)
                {
                    item.OperatorKvid = operatorKvid.Value;
                    item.AddOnlyProperties(o => o.OperatorKvid);
                }
                item.Category = $"发送失败";
                item.AddOnlyProperties(o => o.Category);
                item.AddOnlyProperties(o => o.Remark);
                conn.UpdateOnly(item);
                rtns.Add(item);

                var log = new EntityLog<G>();
                log.OwnerKvid = item.OwnerKvid;
                log.BizId = item.OwnerName;
                log.BizKvid = item.OwnerKvid;
                log.BizType = typeof(G).FullName;
                log.Type = "电子邮箱";
                log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                log.Summary = "电子邮箱发送失败";
                log.Remark = $"[发送失败] 发送{item.ToAddress}失败原因:{item.Remark}. 时间:{DateTime.Now:yyyy-MM-dd HH:mm:ss}!";
                conn.Insert(log);
            }

            return rtns;
        }

        public static List<EmailPool<G>> SendCancel<G>(this List<EmailPool<G>> items, string operatorName = null, Guid? operatorKvid = null, bool isDelete=false, IDbConnection conn = null)
        {
            var rtns = new List<EmailPool<G>>();
            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<EmailPool<G>>();
            if (operatorName.IsNullOrEmpty())
            {
                operatorName = KiviiContext.CurrentMember.FullName;
                operatorKvid = KiviiContext.CurrentMember.Kvid;
            }
            foreach (var item in items)
            {
                if (isDelete)
                {
                    item.Status = -1;
                    item.AddOnlyProperties(o => o.Status);
                    conn.UpdateOnly(item);
                    rtns.Add(item);
                    continue;
                }
                if (item.IsSended) continue;
                item.IsSended = true;
                item.AddOnlyProperties(o => o.IsSended);
                item.OperatorName = operatorName;
                item.AddOnlyProperties(o => o.OperatorName);
                if (operatorKvid != null)
                {
                    item.OperatorKvid = operatorKvid.Value;
                    item.AddOnlyProperties(o => o.OperatorKvid);
                }
                item.Category = $"取消发送";
                item.AddOnlyProperties(o => o.Category);
                conn.UpdateOnly(item);
                rtns.Add(item);

                var log = new EntityLog<G>();
                log.OwnerKvid = item.OwnerKvid;
                log.BizId = item.OwnerName;
                log.BizKvid = item.OwnerKvid;
                log.BizType = typeof(G).FullName;
                log.Type = "电子邮箱";
                log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                log.Summary = "取消电子邮箱发送";
                log.Remark = $"[取消发送] 操作人:{KiviiContext.CurrentMember.FullName}. 时间:{DateTime.Now:yyyy-MM-dd HH:mm:ss}!";
                conn.Insert(log);
            }

            return rtns;
        }



    }
}
