﻿using Kivii.Configuration;
using Kivii.EmailPools.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Net.Mail;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.EmailPools.Transforms
{
    [Api(Description = "邮箱池创建")]
    //[RequiresAnyRole(MemberRoles.Everyone)]
    public class EmailPoolCreate : RestfulExecutionGeneric<EmailPool>
    {
        public EmailPool Item { get; set; }
        public List<EmailPool> Items { get; set; }

        public override object OnExecutionGeneric<G>(IRequest req, IResponse res)
        {
            (Item == null && Items.IsNullOrEmpty()).ThrowIfTrue("Need Item Or Items!");

            if (Items.IsNullOrEmpty()) Items = new List<EmailPool>();
            if (Item != null) Items.Add(Item);
            Console.WriteLine("-----------------------EmailCreateStart---------------------------------");
            var conn = KiviiContext.GetOpenedDbConnection<EmailPool<G>>();
            var rtns = new RestfulCreateResponse<EmailPool<G>>();
            rtns.Results = new List<EmailPool<G>>();

            var results = Items.Accept<G>(conn);
            rtns.Results.AddRange(results);
            return rtns;
        }
    }

    [Api(Description = "邮箱池更新")]
    //[RequiresAnyRole(MemberRoles.Everyone)]
    public class EmailPoolUpdate : RestfulExecutionGeneric<EmailPool>
    {
        public EmailPool Item { get; set; }

        public List<EmailPool> Items { get; set; }

        public override object OnExecutionGeneric<G>(IRequest req, IResponse res)
        {
            (Item == null && Items.IsNullOrEmpty()).ThrowIfTrue("Need Item Or Items!");

            if (Items.IsNullOrEmpty()) Items = new List<EmailPool>();
            if (Item != null) Items.Add(Item);

            var conn = KiviiContext.GetOpenedDbConnection<EmailPool<G>>();

            var emailPools = conn.SelectByIds<EmailPool<G>>(Items.ConvertAll(o => o.Kvid));
            emailPools.ThrowIfNullOrEmpty("未找到要更新的数据!");
            if (emailPools.Exists(o => o.IsSended))
            {
                throw new Exception("存在已发送的数据,无法更新数据!");
            }

            var rtns = new RestfulUpdateResponse<EmailPool<G>>();
            rtns.Results = new List<EmailPool<G>>();

            foreach (var item in Items)
            {
                var emailPool = emailPools.FirstOrDefault(o => o.Kvid == item.Kvid);
                if (emailPool != null)
                {
                    if (item.OnlyPropertiesIsExist(o => o.ToAddress))
                    {
                        try
                        {
                            var to = new MailAddress(item.ToAddress, item.ToName);
                        }
                        catch
                        {
                            throw new Exception($"收件地址{item.ToAddress},格式错误");
                        }
                        emailPool.ToAddress = item.ToAddress;
                        emailPool.AddOnlyProperties(o => o.ToAddress);
                        conn.UpdateOnly(emailPool);
                        rtns.Results.Add(emailPool);
                    }
                }
            }
            return rtns;
        }
    }

    [Api(Description = "邮箱池查询")]
    //[RequiresAnyRole(MemberRoles.Everyone)]
    public class EmailPoolQuery : RestfulExecutionGeneric<EmailPool>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion
        public override object OnExecutionGeneric<G>(IRequest req, IResponse res)
        {
            Console.WriteLine("---------------Start:Query----------------");
            var conn = KiviiContext.GetOpenedDbConnection<EmailPool<G>>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<EmailPool<G>>();
            request = request.PopulateWith(this);
            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);
            Console.WriteLine($"---------------Query:{rtns.Total}----------------");
            Console.WriteLine("---------------End:Query----------------");
            return rtns;
        }
    }

    [Api(Description = "未发送邮箱池")]
    //[RequiresAnyRole(MemberRoles.Everyone)]
    public class EmailPoolPreSending : RestfulExecutionGeneric<EmailPool>
    {
        public override object OnExecutionGeneric<G>(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<EmailPool<G>>();

            var rtns = new RestfulQueryResponse<EmailPool<G>>();
            rtns.Results = new List<EmailPool<G>>();
            rtns.Results = conn.GetPreSending<G>();

            rtns.Total = rtns.Results.Count;

            return rtns;
        }
    }

    [Api(Description = "标记发送完成")]
    //[RequiresAnyRole(MemberRoles.Everyone)]
    public class EmailPoolComplete : RestfulExecutionGeneric<EmailPool>
    {
        public List<Guid> Kvids { get; set; }
        public string OperatorName { get; set; }
        public Guid OperatorKvid { get; set; }

        public override object OnExecutionGeneric<G>(IRequest req, IResponse res)
        {
            Kvids.ThrowIfNullOrEmpty("Need Kvids");

            var conn = KiviiContext.GetOpenedDbConnection<EmailPool<G>>();
            var pools = conn.SelectByIds<EmailPool<G>>(Kvids);
            pools.ThrowIfNullOrEmpty($"EmailPool Not Found:Kvid={Kvids}");

            var rtns = new RestfulExecutionResponse<EmailPool<G>>();
            rtns.Results = new List<EmailPool<G>>();
            rtns.Results = pools.SendComplete<G>(OperatorName, OperatorKvid, conn);

            return rtns;
        }
    }

    [Api(Description = "标记发送失败")]
    //[RequiresAnyRole(MemberRoles.Everyone)]
    public class EmailPoolFail : RestfulExecutionGeneric<EmailPool>
    {
        public List<EmailPool> Items { get; set; }

        public string OperatorName { get; set; }
        public Guid OperatorKvid { get; set; }

        public override object OnExecutionGeneric<G>(IRequest req, IResponse res)
        {
            Items.ThrowIfNullOrEmpty("Need Kvids");

            var conn = KiviiContext.GetOpenedDbConnection<EmailPool<G>>();
            var pools = conn.SelectByIds<EmailPool<G>>(Items.ConvertAll(p => p.Kvid));
            pools.ThrowIfNullOrEmpty($"EmailPools Not Found!");

            var rtns = new RestfulExecutionResponse<EmailPool<G>>();
            rtns.Results = new List<EmailPool<G>>();

            foreach (var item in pools)
            {
                var current = Items.FirstOrDefault(o => o.Kvid == item.Kvid);
                if (current == null) continue;
                item.Remark = current.Remark;
            }
            rtns.Results = pools.SendFail<G>(OperatorName, OperatorKvid, conn);

            return rtns;
        }
    }

    [Api(Description = "取消发送")]
    //[RequiresAnyRole(MemberRoles.Everyone)]
    public class EmailPoolCancel : RestfulExecutionGeneric<EmailPool>
    {
        public List<Guid> Kvids { get; set; }
        public bool IsDelete { get; set; } = false;
        public string OperatorName { get; set; }
        public Guid OperatorKvid { get; set; }
        public override object OnExecutionGeneric<G>(IRequest req, IResponse res)
        {
            Kvids.ThrowIfNullOrEmpty("Need Kvids");

            var conn = KiviiContext.GetOpenedDbConnection<EmailPool<G>>();
            var pools = conn.SelectByIds<EmailPool<G>>(Kvids);
            pools.ThrowIfNullOrEmpty($"EmailPools Not Found!");

            var rtns = new RestfulExecutionResponse<EmailPool<G>>();
            rtns.Results = new List<EmailPool<G>>();
            rtns.Results = pools.SendCancel<G>(OperatorName, OperatorKvid, IsDelete, conn);

            return rtns;
        }
    }
}
